@@ -104,6 +104,86 @@ if ($do==1) {
                "\n\n";
          &Net::FullAuto::FA_Core::cleanup;
       }
+      # http://www.iis.net/learn/install/installing-iis-85/
+      #      installing-iis-85-on-windows-server-2012-r2
+      # https://technet.microsoft.com/en-us/library/hh831475.aspx
+      # http://twiki.org/cgi-bin/view/TWiki/TWikiOnWindowsIISCygwin
+      # https://www.iis.net/configreference/system.webserver/fastcgi
+      ($stdout,$stderr)=$handle->cmd("DISM.EXE /enable-feature /all ".
+         "/online /featureName:IIS-WebServerRole /featureName:IIS-WebServer ".
+         "/featureName:IIS-CommonHttpFeatures /featureName:IIS-StaticContent ".
+         "/featureName:IIS-DefaultDocument /featureName:IIS-DirectoryBrowsing ".
+         "/featureName:IIS-HttpErrors /featureName:IIS-HttpRedirect ".
+         "/featureName:IIS-ApplicationDevelopment /featureName:IIS-ASPNET ".
+         "/featureName:IIS-NetFxExtensibility /featureName:IIS-ASPNET45 ".
+         "/featureName:IIS-NetFxExtensibility45 /featureName:IIS-ASP ".
+         "/featureName:IIS-CGI /featureName:IIS-ISAPIExtensions ".
+         "/featureName:IIS-ISAPIFilter /featureName:IIS-ServerSideIncludes ".
+         "/featureName:IIS-HealthAndDiagnostics /featureName:IIS-HttpLogging ".
+         "/featureName:IIS-LoggingLibraries /featureName:IIS-RequestMonitor ".
+         "/featureName:IIS-HttpTracing /featureName:IIS-CustomLogging ".
+         "/featureName:IIS-ODBCLogging /featureName:IIS-Security ".
+         "/featureName:IIS-BasicAuthentication ".
+         "/featureName:IIS-WindowsAuthentication ".
+         "/featureName:IIS-DigestAuthentication ".
+         "/featureName:IIS-ClientCertificateMappingAuthentication ".
+         "/featureName:IIS-IISCertificateMappingAuthentication ".
+         "/featureName:IIS-URLAuthorization /featureName:IIS-RequestFiltering ".
+         "/featureName:IIS-IPSecurity /featureName:IIS-Performance ".
+         "/featureName:IIS-HttpCompressionStatic ".
+         "/featureName:IIS-HttpCompressionDynamic /featureName:IIS-WebDAV ".
+         "/featureName:IIS-WebServerManagementTools ".
+         "/featureName:IIS-ManagementScriptingTools ".
+         "/featureName:IIS-ManagementService ".
+         "/featureName:IIS-IIS6ManagementCompatibility ".
+         "/featureName:IIS-Metabase /featureName:IIS-WMICompatibility ".
+         "/featureName:IIS-LegacyScripts /featureName:NetFx4Extended-ASPNET45 ".
+         "/featureName:IIS-ApplicationInit /featureName:IIS-WebSockets ".
+         "/featureName:IIS-CertProvider /featureName:IIS-ManagementConsole ".
+         "/featureName:IIS-LegacySnapIn",'__display__');
+      my $appcmd="$ENV{WINDIR}/SYSTEM32/inetsrv";
+      ($appcmd,$stderr)=$handle->cmd("cygpath -u $appcmd");
+      my $cygloc=$handle->cmd("cygpath -w ~");
+      $cygloc.="\\FullAutoAPI";
+      $cygloc=~s/\\/\\\\/g;
+      ($stdout,$stderr)=$handle->cwd($appcmd);
+      ($stdout,$stderr)=$handle->cmd("appcmd add site /name: FullAutoAPI ".
+         "/id:2 /physicalPath: $cygloc\\root /bindings:http/*:5000",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd("appcmd set config -section:".
+         "system.webServer/fastCgi /+[\"fullpath=\'$cygloc\\script\\".
+         "CGI_script\',arguments=\'$cygloc\\script\\fullautoapi_fastcgi.pl ".
+         "-e\',maxInstances=\'4\',idleTimeout=\'300\',activityTimeout=\'30\',".
+         "requestTimeout=\90\',instanceMaxRequests=\'1000\',protocol=".
+         "\'NamedPipe\',flushNamedPipe=\'False\']" /commit:apphost",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd("appcmd set config -section:".
+         "\"FullAutoAPI\" -section:system.webServer/handlers /+\"[name=".
+         "\'FullAutoAPI\',path=\'*\',verb=\'GET,HEAD,POST\',modules=".
+         "\'FastCgiModule\',scriptProcessor=\'$cygloc\\script\\".
+         "CGI_Script|$cygloc\\script\\fullautoapi_fastcgi.pl -e\',". 
+         "resourceType=\'Unspecified\',requireAccess=\'Script\']\" ".
+         "/commit:apphost",'__display__');
+      ($stdout,$stderr)=$handle->cmd("touch CGI_Script");
+      #
+      # echo-ing/streaming files over ssh can be tricky. Use echo -e
+      #          and replace these characters with thier HEX
+      #          equivalents (use an external editor for quick
+      #          search and replace - and paste back results.
+      #          use copy/paste or cat file and copy/paste results.):
+      #
+      #          !  -   \\x21
+      #          "  -   \\x22
+      #          $  -   \\x24
+      #
+      my $content=<<END;
+#\\x21/bin/sh
+cd bin
+/bin/perl -wT \\x241
+END
+      ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > CGI_Script");
+      ($stdout,$stderr)=$handle->cwd("~");
+      #($stdout,$stderr)=$handle->cmd("iisreset /start",'__display__');
    }
    ###############
    ## RABBITMQ
@@ -207,14 +287,10 @@ if ($do==1) {
    ($stdout,$stderr)=$handle->cmd("mkdir -vp FullAutoAPI/deps",'__display__');
    ($stdout,$stderr)=$handle->cmd("mkdir -vp FullAutoAPI/root/static/images",
       '__display__');
-$Net::FullAuto::FA_Core::debug=1;
    ($stdout,$stderr)=$handle->cmd("perl -e \'use CPAN;".
       "CPAN::HandleConfig-\>load;print \$CPAN::Config-\>{build_dir}\'");
-$Net::FullAuto::FA_Core::debug=0;
-print "STDOUT for BUILDDIR=$stdout and STDERR FOR PERL=$stderr\n";sleep 20;
    my $builddir=$stdout;
    my $fa_ver=$Net::FullAuto::VERSION;
-print "FAVER=$fa_ver\n";
    ($stdout,$stderr)=$handle->cmd(
       "ls -1t $builddir | grep Net-FullAuto-$fa_ver");
    my @lstmp=split /\n/,$stdout;
@@ -1005,7 +1081,8 @@ END
        "    +CatalystX::SimpleLogin";
    ($stdout,$stderr)=$handle->cmd(
        "${sudo}sed -i '/    Static::Simple/a$ad' $pm_path");
-   $ad="    authentication => {%NL%".
+   $ad="    use_request_uri_for_path => 1,%NL%".
+       "    authentication => {%NL%".
        "       default_realm => %SQ%users%SQ%,%NL%".
        "       realms        => {%NL%".
        "          users => {%NL%".
@@ -1038,8 +1115,8 @@ END
    ($stdout,$stderr)=$handle->cmd(
        "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $pm_path");
    ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" $pm_path");
-   ($stdout,$stderr)=$handle->cmd("mkdir -vp root/src/login",'__display__');
-   ($stdout,$stderr)=$handle->cmd("touch root/src/login/login.tt2",
+   ($stdout,$stderr)=$handle->cmd("mkdir -vp root/login",'__display__');
+   ($stdout,$stderr)=$handle->cmd("touch root/login/login.tt",
       '__display__');
    my $content=<<'ENDD';
 [% META title = 'Welcome to FullAuto: Please Log In' %]
@@ -1198,7 +1275,7 @@ END
    #          and replace these characters with thier HEX
    #          equivalents (use an external editor for quick
    #          search and replace - and paste back results.
-   #          use copy/paste or cat file and copy/paster results.):
+   #          use copy/paste or cat file and copy/paste results.):
    #
    #          !  -   \\x21
    #          "  -   \\x22
@@ -1281,8 +1358,15 @@ END
       "./script/fullautoapi_create.pl controller User",'__display__');
    ($stdout,$stderr)=$handle->cmd("mv lib/FullAutoAPI/Controller/User.pm ".
       "lib/FullAutoAPI/Controller/User.bak");
+   my $view_path='./FullAutoAPI/View/TT.pm';
    ($stdout,$stderr)=$handle->cmd(
       "./script/fullautoapi_create.pl view TT TT",'__display__');
+   $ad='    WRAPPER => %SQ%wrapper.tt%SQ%,%NL%';
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}sed -i \'/render_die/a$ad\' $view_path");
+   ($stdout,$stderr)=$handle->cmd( # bash shell specific
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $view_path");
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" $view_path");
    ($stdout,$stderr)=$handle->cmd(
       "mkdir -vp root/static/jquery",'__display__');
    ($stdout,$stderr)=$handle->cwd('root/static/jquery');
@@ -1307,6 +1391,17 @@ END
    ($stdout,$stderr)=$handle->cwd('examples');
    ($stdout,$stderr)=$handle->cmd('cp -Rv resources ../..','__display__');
    ($stdout,$stderr)=$handle->cwd('../../..');
+   #
+   # echo-ing/streaming files over ssh can be tricky. Use echo -e
+   #          and replace these characters with thier HEX
+   #          equivalents (use an external editor for quick
+   #          search and replace - and paste back results.
+   #          use copy/paste or cat file and copy/paste results.):
+   #
+   #          !  -   \\x21
+   #          "  -   \\x22
+   #          $  -   \\x24
+   #
    $content=<<END;
 package FullAutoAPI::Controller::User;
  
@@ -1346,6 +1441,50 @@ __PACKAGE__->config(
     },
 );
 
+sub base : Chained('/base') PathPrefix CaptureArgs(0) {}
+
+sub admin : Chained('base') PathPart('') CaptureArgs(0) Does('ACL') RequiresRole('admin') ACLDetachTo('denied') {}
+
+sub change_password : Chained('base') PathPart('change_password') Args(0) {
+    my (\\x24self, \\x24c) = @_;
+
+    my \\x24form = FullAutoAPI::Form::ChangePassword->new;
+
+    \\x24c->stash(form => \\x24form);
+
+    return unless \\x24form->process(
+        user   => \\x24c->user,
+        params => \\x24c->req->body_parameters,
+    );
+
+    \\x24c->user->update({
+        password         => \\x24form->field('new_password')->value,
+        password_expires => undef,
+    });
+
+    \\x24c->res->redirect(\\x24c->uri_for('/books/list', {
+        status_msg => 'Password changed successfully'
+    }));
+}
+
+sub profile : Chained('base') PathPart('profile') Args(0) {
+    my (\\x24self, \\x24c) = @_;
+
+    my \\x24form = MyApp::Form::UserProfile->new;
+
+    \\x24c->stash(form => \\x24form);
+
+    return unless \\x24form->process(
+        schema  => \\x24c->model('DB')->schema,
+        item_id => \\x24c->user->id,
+        params  => \\x24c->req->body_parameters,
+    );
+
+    \\x24c->res->redirect(\\x24c->uri_for('/user', {
+        status_msg => 'Profile Updated'
+    }));
+}
+
 sub user_list : Path('/user') :Args(0) : ActionClass('REST') { }
 
 sub user_list_GET {
@@ -1511,7 +1650,20 @@ END
    ($stdout,$stderr)=$handle->cmd(
       "${sudo}sed -i '/index :Path :/,+6d' ./Root.pm");
    $ad='%NL%sub base : Chained(%SQ%/login/required%SQ%) PathPrefix '.
-       'CaptureArgs(0) {}%NL%';
+       'CaptureArgs(0) {%NL%'.
+       '    my ($self, $c) = @_;%NL%'.
+       '%NL%'.
+       '    if ($c->action ne $c->controller(%SQ%Users%SQ%)->action_for('.
+       '%SQ%change_password%SQ%)%NL%'.
+       '        && $c->user_exists%NL%'.
+       '        && $c->user->password_expires%NL%'.
+       '        && $c->user->password_expires <= DateTime->now)%NL%'.
+       '    {%NL%        '.
+       '$c->res->redirect($c->uri_for(%SQ%/user/change_password%SQ%, {%NL%'.
+       '            status_msg => %SQ%Password Expired%SQ%%NL%'.
+       '        }));%NL%'.
+       '        $c->detach;%NL%'.
+       '    }%NL%';
    ($stdout,$stderr)=$handle->cmd(
       "${sudo}sed -i \'/index : Private/i$ad\' ./Root.pm");
    ($stdout,$stderr)=$handle->cmd(
@@ -1520,6 +1672,122 @@ END
    ($stdout,$stderr)=$handle->cmd( # bash shell specific
       "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ./Root.pm");
    ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" ./Root.pm");
+   ($stdout,$stderr)=$handle->cwd("..");
+   ($stdout,$stderr)=$handle->cmd("mkdir -vp Form");
+   ($stdout,$stderr)=$handle->cwd("Form");
+   $content=<<END;
+package FullAutoAPI::Form::AddUser;
+
+use HTML::FormHandler::Moose;
+extends 'FullAutoAPI::Form::EditUser';
+use namespace::autoclean;
+
+has_field 'username' => (
+    type     => 'Text',
+    label    => 'User name',
+    required => 1,
+);
+
+__PACKAGE__->meta->make_immutable;
+
+1;
+END
+   ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > AddUser.pm");
+   $content=<<END;
+package FullAutoAPI::Form::ChangePassword;
+
+use HTML::FormHandler::Moose;
+extends 'HTML::FormHandler';
+use namespace::autoclean;
+use Method::Signatures::Simple;
+
+has user => (is => 'rw');
+
+has_field 'current_password' => (
+   type     => 'Password',
+   label    => 'Current Password',
+   required => 1,
+);
+
+method validate_current_password(\\x24field) {
+    \\x24field->add_error('Incorrect password')
+        if not \\x24self->user->check_password(\\x24field->value);
+}
+
+has_field 'new_password' => (
+    type      => 'Password',
+    label     => 'New Password',
+    required  => 1,
+    minlength => 5,
+);
+
+after validate => method {
+    if (\\x24self->field('new_password')->value eq
+            \\x24self->field('current_password')->value )
+    {
+        \\x24self->field('new_password')
+            ->add_error('Must be different from current password');
+    }
+};
+
+has_field 'new_password_conf' => (
+   type           => 'PasswordConf',
+   label          => 'New Password (again)',
+   password_field => 'new_password',
+   required       => 1,
+   minlength      => 5,
+);
+
+has_field submit => (type => 'Submit', value => 'Change');
+
+__PACKAGE__->meta->make_immutable;
+
+1;
+END
+   ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > ChangePassword.pm");
+   $content=<<END;
+package FullAutoAPI::Form::EditUser;
+
+use HTML::FormHandler::Moose;
+extends 'FullAutoAPI::Form::UserProfile';
+use namespace::autoclean;
+
+has_field 'roles' => (
+    type         => 'Multiple',
+    widget       => 'checkbox_group',
+    label_column => 'name',
+    label        => '',
+);
+
+__PACKAGE__->meta->make_immutable;
+
+1;
+END
+   ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > EditUser.pm");
+   $content=<<END;
+package FullAutoAPI::Form::UserProfile;
+
+use HTML::FormHandler::Moose;
+extends 'HTML::FormHandler::Model::DBIC';
+use namespace::autoclean;
+
+has '+item_class' => (default => 'User');
+
+has_field 'name'          => ( type => 'Text',  required => 1 );
+has_field 'email_address' => ( type => 'Email', required => 1 );
+has_field 'phone_number'  => ( type => 'Text' );
+has_field 'mail_address'  => ( type => 'Text' );
+
+has_field submit => (
+   type  => 'Submit',
+   value => 'Update'
+);
+
+__PACKAGE__->meta->make_immutable;
+
+1;
+END
+   ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > UserProfile.pm");
    ($stdout,$stderr)=$handle->cmd("chmod -v 644 User.pm",'__display__');
    ($stdout,$stderr)=$handle->cwd('../../../root/static');
    ($stdout,$stderr)=$handle->cmd(
@@ -1554,7 +1822,7 @@ END
    #          and replace these characters with thier HEX
    #          equivalents (use an external editor for quick
    #          search and replace - and paste back results.
-   #          use copy/paste or cat file and copy/paster results.):
+   #          use copy/paste or cat file and copy/paste results.):
    #
    #          !  -   \\x21
    #          "  -   \\x22
